@@ -34,19 +34,19 @@
 namespace NArchive {
 namespace NWim {
 
-static int inline GetLog(UInt32 num)
+static bool inline GetLog_val_min_dest(const UInt32 val, unsigned i, unsigned &dest)
 {
-  for (int i = 0; i < 32; i++)
-    if (((UInt32)1 << i) == num)
-      return i;
-  return -1;
-}
-
-
-CUnpacker::~CUnpacker()
-{
-  if (lzmsDecoder)
-    delete lzmsDecoder;
+  UInt32 v = (UInt32)1 << i;
+  for (; i < 32; i++)
+  {
+    if (v == val)
+    {
+      dest = i;
+      return true;
+    }
+    v += v;
+  }
+  return false;
 }
 
 
@@ -64,25 +64,27 @@ HRESULT CUnpacker::UnpackChunk(
   }
   else if (method == NMethod::kLZX)
   {
-    if (!lzxDecoder)
-    {
-      lzxDecoderSpec = new NCompress::NLzx::CDecoder(true);
-      lzxDecoder = lzxDecoderSpec;
-    }
+    lzxDecoder.Create_if_Empty();
+    lzxDecoder->Set_WimMode(true);
   }
   else if (method == NMethod::kLZMS)
   {
-    if (!lzmsDecoder)
-      lzmsDecoder = new NCompress::NLzms::CDecoder();
+    lzmsDecoder.Create_if_Empty();
   }
   else
     return E_NOTIMPL;
 
   const size_t chunkSize = (size_t)1 << chunkSizeBits;
-  
-  unpackBuf.EnsureCapacity(chunkSize);
-  if (!unpackBuf.Data)
-    return E_OUTOFMEMORY;
+
+  {
+    const unsigned
+        kAdditionalOutputBufSize = MyMax(NCompress::NLzx::
+        kAdditionalOutputBufSize,        NCompress::NXpress::
+        kAdditionalOutputBufSize);
+    unpackBuf.EnsureCapacity(chunkSize + kAdditionalOutputBufSize);
+    if (!unpackBuf.Data)
+      return E_OUTOFMEMORY;
+  }
   
   HRESULT res = S_FALSE;
   size_t unpackedSize = 0;
@@ -95,30 +97,32 @@ HRESULT CUnpacker::UnpackChunk(
   }
   else if (inSize < chunkSize)
   {
-    packBuf.EnsureCapacity(chunkSize);
+    const unsigned kAdditionalInputSize = 32;
+    packBuf.EnsureCapacity(chunkSize + kAdditionalInputSize);
     if (!packBuf.Data)
       return E_OUTOFMEMORY;
     
     RINOK(ReadStream_FALSE(inStream, packBuf.Data, inSize))
+    memset(packBuf.Data + inSize, 0xff, kAdditionalInputSize);
 
     TotalPacked += inSize;
     
     if (method == NMethod::kXPRESS)
     {
-      res = NCompress::NXpress::Decode(packBuf.Data, inSize, unpackBuf.Data, outSize);
+      res = NCompress::NXpress::Decode_WithExceedWrite(packBuf.Data, inSize, unpackBuf.Data, outSize);
       if (res == S_OK)
         unpackedSize = outSize;
     }
     else if (method == NMethod::kLZX)
     {
-      res = lzxDecoderSpec->SetExternalWindow(unpackBuf.Data, chunkSizeBits);
+      res = lzxDecoder->Set_ExternalWindow_DictBits(unpackBuf.Data, chunkSizeBits);
       if (res != S_OK)
         return E_NOTIMPL;
-      lzxDecoderSpec->KeepHistoryForNext = false;
-      lzxDecoderSpec->SetKeepHistory(false);
-      res = lzxDecoderSpec->Code(packBuf.Data, inSize, (UInt32)outSize);
-      unpackedSize = lzxDecoderSpec->GetUnpackSize();
-      if (res == S_OK && !lzxDecoderSpec->WasBlockFinished())
+      lzxDecoder->Set_KeepHistoryForNext(false);
+      lzxDecoder->Set_KeepHistory(false);
+      res = lzxDecoder->Code_WithExceedReadWrite(packBuf.Data, inSize, (UInt32)outSize);
+      unpackedSize = lzxDecoder->GetUnpackSize();
+      if (res == S_OK && !lzxDecoder->WasBlockFinished())
         res = S_FALSE;
     }
     else
@@ -158,26 +162,21 @@ HRESULT CUnpacker::Unpack2(
 {
   if (!resource.IsCompressed() && !resource.IsSolid())
   {
-    if (!copyCoder)
-    {
-      copyCoderSpec = new NCompress::CCopyCoder;
-      copyCoder = copyCoderSpec;
-    }
+    copyCoder.Create_if_Empty();
 
-    CLimitedSequentialInStream *limitedStreamSpec = new CLimitedSequentialInStream();
-    CMyComPtr<ISequentialInStream> limitedStream = limitedStreamSpec;
-    limitedStreamSpec->SetStream(inStream);
+    CMyComPtr2_Create<ISequentialInStream, CLimitedSequentialInStream> limitedStream;
+    limitedStream->SetStream(inStream);
     
     RINOK(InStream_SeekSet(inStream, resource.Offset))
     if (resource.PackSize != resource.UnpackSize)
       return S_FALSE;
 
-    limitedStreamSpec->Init(resource.PackSize);
+    limitedStream->Init(resource.PackSize);
     TotalPacked += resource.PackSize;
     
-    HRESULT res = copyCoder->Code(limitedStream, outStream, NULL, NULL, progress);
+    HRESULT res = copyCoder.Interface()->Code(limitedStream, outStream, NULL, NULL, progress);
     
-    if (res == S_OK && copyCoderSpec->TotalSize != resource.UnpackSize)
+    if (res == S_OK && copyCoder->TotalSize != resource.UnpackSize)
       res = S_FALSE;
     return res;
   }
@@ -366,24 +365,13 @@ HRESULT CUnpacker::Unpack2(
 HRESULT CUnpacker::Unpack(IInStream *inStream, const CResource &resource, const CHeader &header, const CDatabase *db,
     ISequentialOutStream *outStream, ICompressProgressInfo *progress, Byte *digest)
 {
-  COutStreamWithSha1 *shaStreamSpec = NULL;
-  CMyComPtr<ISequentialOutStream> shaStream;
-  
+  CMyComPtr2_Create<ISequentialOutStream, COutStreamWithSha1> shaStream;
   // outStream can be NULL, so we use COutStreamWithSha1 even if sha1 is not required
-  // if (digest)
-  {
-    shaStreamSpec = new COutStreamWithSha1();
-    shaStream = shaStreamSpec;
-    shaStreamSpec->SetStream(outStream);
-    shaStreamSpec->Init(digest != NULL);
-    outStream = shaStream;
-  }
-  
-  HRESULT res = Unpack2(inStream, resource, header, db, outStream, progress);
-  
+  shaStream->SetStream(outStream);
+  shaStream->Init(digest != NULL);
+  const HRESULT res = Unpack2(inStream, resource, header, db, shaStream, progress);
   if (digest)
-    shaStreamSpec->Final(digest);
-  
+    shaStream->Final(digest);
   return res;
 }
 
@@ -394,21 +382,16 @@ HRESULT CUnpacker::UnpackData(IInStream *inStream,
     CByteBuffer &buf, Byte *digest)
 {
   // if (resource.IsSolid()) return E_NOTIMPL;
-
   UInt64 unpackSize64 = resource.UnpackSize;
   if (db)
     unpackSize64 = db->Get_UnpackSize_of_Resource(resource);
-
-  size_t size = (size_t)unpackSize64;
+  const size_t size = (size_t)unpackSize64;
   if (size != unpackSize64)
     return E_OUTOFMEMORY;
-
   buf.Alloc(size);
 
-  CBufPtrSeqOutStream *outStreamSpec = new CBufPtrSeqOutStream();
-  CMyComPtr<ISequentialOutStream> outStream = outStreamSpec;
-  outStreamSpec->Init((Byte *)buf, size);
-
+  CMyComPtr2_Create<ISequentialOutStream, CBufPtrSeqOutStream> outStream;
+  outStream->Init((Byte *)buf, size);
   return Unpack(inStream, resource, header, db, outStream, NULL, digest);
 }
 
@@ -900,10 +883,8 @@ HRESULT CHeader::Parse(const Byte *p, UInt64 &phySize)
     ChunkSizeBits = kChunkSizeBits;
     if (ChunkSize != 0)
     {
-      const int log = GetLog(ChunkSize);
-      if (log < 12)
+      if (!GetLog_val_min_dest(ChunkSize, 12, ChunkSizeBits))
         return S_FALSE;
-      ChunkSizeBits = (unsigned)log;
     }
   }
 
@@ -1314,23 +1295,21 @@ HRESULT CDatabase::FillAndCheck(const CObjectVector<CVolume> &volumes)
             return S_FALSE;
 
           const UInt32 solidChunkSize = GetUi32(header + 8);
-          const int log = GetLog(solidChunkSize);
-          if (log < 8 || log > 31)
+          if (!GetLog_val_min_dest(solidChunkSize, 8, ss.ChunkSizeBits))
             return S_FALSE;
-          ss.ChunkSizeBits = (unsigned)log;
           ss.Method = (Int32)GetUi32(header + 12);
           
-          UInt64 numChunks64 = (ss.UnpackSize + (((UInt32)1 << ss.ChunkSizeBits) - 1)) >> ss.ChunkSizeBits;
-          UInt64 sizesBufSize64 = 4 * numChunks64;
+          const UInt64 numChunks64 = (ss.UnpackSize + (((UInt32)1 << ss.ChunkSizeBits) - 1)) >> ss.ChunkSizeBits;
+          const UInt64 sizesBufSize64 = 4 * numChunks64;
           ss.HeadersSize = kSolidHeaderSize + sizesBufSize64;
-          size_t sizesBufSize = (size_t)sizesBufSize64;
+          const size_t sizesBufSize = (size_t)sizesBufSize64;
           if (sizesBufSize != sizesBufSize64)
             return E_OUTOFMEMORY;
           sizesBuf.AllocAtLeast(sizesBufSize);
           
           RINOK(ReadStream_FALSE(inStream, sizesBuf, sizesBufSize))
           
-          size_t numChunks = (size_t)numChunks64;
+          const size_t numChunks = (size_t)numChunks64;
           ss.Chunks.Alloc(numChunks + 1);
 
           UInt64 offset = 0;
@@ -1439,7 +1418,7 @@ HRESULT CDatabase::FillAndCheck(const CObjectVector<CVolume> &volumes)
   
   {
     {
-      const CStreamInfo *streams = &DataStreams.Front();
+      const CStreamInfo *streams = DataStreams.ConstData();
 
       if (IsOldVersion)
       {
@@ -1481,7 +1460,7 @@ HRESULT CDatabase::FillAndCheck(const CObjectVector<CVolume> &volumes)
           hash += (item.IsAltStream ? 0x8 : 0x10);
           UInt32 id = GetUi32(hash);
           if (id != 0)
-            item.StreamIndex = FindId(&DataStreams.Front(), sortedByHash, id);
+            item.StreamIndex = FindId(DataStreams.ConstData(), sortedByHash, id);
         }
       }
       /*
@@ -1495,7 +1474,7 @@ HRESULT CDatabase::FillAndCheck(const CObjectVector<CVolume> &volumes)
         hash += (item.IsAltStream ? 0x10 : 0x40);
         if (!IsEmptySha(hash))
         {
-          item.StreamIndex = FindHash(&DataStreams.Front(), sortedByHash, hash);
+          item.StreamIndex = FindHash(DataStreams.ConstData(), sortedByHash, hash);
         }
       }
     }
@@ -1518,7 +1497,7 @@ HRESULT CDatabase::FillAndCheck(const CObjectVector<CVolume> &volumes)
     
     for (i = 0; i < Items.Size(); i++)
     {
-      int streamIndex = Items[i].StreamIndex;
+      const int streamIndex = Items[i].StreamIndex;
       if (streamIndex >= 0)
         refCounts[streamIndex]++;
     }
@@ -1773,13 +1752,12 @@ static bool ParseNumber32(const AString &s, UInt32 &res)
 
 static bool ParseTime(const CXmlItem &item, FILETIME &ft, const char *tag)
 {
-  int index = item.FindSubTag(tag);
-  if (index >= 0)
+  const CXmlItem *timeItem = item.FindSubTag_GetPtr(tag);
+  if (timeItem)
   {
-    const CXmlItem &timeItem = item.SubItems[index];
     UInt32 low = 0, high = 0;
-    if (ParseNumber32(timeItem.GetSubStringForTag("LOWPART"), low) &&
-        ParseNumber32(timeItem.GetSubStringForTag("HIGHPART"), high))
+    if (ParseNumber32(timeItem->GetSubStringForTag("LOWPART"), low) &&
+        ParseNumber32(timeItem->GetSubStringForTag("HIGHPART"), high))
     {
       ft.dwLowDateTime = low;
       ft.dwHighDateTime = high;
