@@ -45,7 +45,7 @@ static const CUInt32PCharPair g_ExtraTypes[] =
 
 void CExtraSubBlock::PrintInfo(AString &s) const
 {
-  for (unsigned i = 0; i < ARRAY_SIZE(g_ExtraTypes); i++)
+  for (unsigned i = 0; i < Z7_ARRAY_SIZE(g_ExtraTypes); i++)
   {
     const CUInt32PCharPair &pair = g_ExtraTypes[i];
     if (pair.Value == ID)
@@ -55,15 +55,15 @@ void CExtraSubBlock::PrintInfo(AString &s) const
       {
         if (Data.Size() >= 1)
         {
-          s += ':';
+          s.Add_Colon();
           const Byte flags = Data[0];
-          if (flags & 1) s += 'M';
-          if (flags & 2) s += 'A';
-          if (flags & 4) s += 'C';
+          if (flags & 1) s.Add_Char('M');
+          if (flags & 2) s.Add_Char('A');
+          if (flags & 4) s.Add_Char('C');
           const UInt32 size = (UInt32)(Data.Size()) - 1;
           if (size % 4 == 0)
           {
-            s += ':';
+            s.Add_Colon();
             s.Add_UInt32(size / 4);
           }
         }
@@ -88,7 +88,7 @@ void CExtraSubBlock::PrintInfo(AString &s) const
     }
   }
   {
-    char sz[32];
+    char sz[16];
     sz[0] = '0';
     sz[1] = 'x';
     ConvertUInt32ToHex(ID, sz + 2);
@@ -291,6 +291,7 @@ bool CItem::IsDir() const
       case NHostOS::kHPFS:
       case NHostOS::kVFAT:
         return true;
+      default: break;
     }
   }
 
@@ -360,6 +361,7 @@ UInt32 CItem::GetWinAttrib() const
         // #endif
       }
       break;
+    default: break;
   }
   if (IsDir()) // test it;
     winAttrib |= FILE_ATTRIBUTE_DIRECTORY;
