@@ -11,18 +11,32 @@
 #include <grp.h>
 #include <pwd.h>
 
+/*
+inclusion of <sys/sysmacros.h> by <sys/types.h> is deprecated since glibc 2.25.
+Since glibc 2.3.3, macros have been aliases for three GNU-specific
+functions: gnu_dev_makedev(), gnu_dev_major(), and gnu_dev_minor()
+
+Warning in GCC:
+In the GNU C Library, "major" is defined by <sys/sysmacros.h>.
+For historical compatibility, it is currently defined by
+<sys/types.h> as well, but we plan to remove this soon.
+To use "major", include <sys/sysmacros.h> directly.
+If you did not intend to use a system-defined macro "major",
+you should undefine it after including <sys/types.h>
+*/
 // for major()/minor():
-#if defined(__FreeBSD__) || defined(BSD)
+#if defined(__APPLE__) || defined(__DragonFly__) || \
+    defined(BSD) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
 #include <sys/types.h>
 #else
 #include <sys/sysmacros.h>
 #endif
 
-#endif
+#endif // _WIN32
 
 #include "../../Windows/FileFind.h"
 
-#ifdef SUPPORT_DEVICE_FILE
+#ifdef Z7_DEVICE_FILE
 #include "../../../C/Alloc.h"
 #include "../../Common/Defs.h"
 #endif
@@ -47,15 +61,15 @@ static inline HRESULT ConvertBoolToHRESULT(bool result)
 }
 
 
-#ifdef SUPPORT_DEVICE_FILE
+#ifdef Z7_DEVICE_FILE
 static const UInt32 kClusterSize = 1 << 18;
 #endif
 
 CInFileStream::CInFileStream():
- #ifdef SUPPORT_DEVICE_FILE
+ #ifdef Z7_DEVICE_FILE
   VirtPos(0),
   PhyPos(0),
-  Buf(0),
+  Buf(NULL),
   BufSize(0),
  #endif
  #ifndef _WIN32
@@ -73,7 +87,7 @@ CInFileStream::CInFileStream():
 
 CInFileStream::~CInFileStream()
 {
-  #ifdef SUPPORT_DEVICE_FILE
+  #ifdef Z7_DEVICE_FILE
   MidFree(Buf);
   #endif
 
@@ -81,11 +95,13 @@ CInFileStream::~CInFileStream()
     Callback->InFileStream_On_Destroy(this, CallbackRef);
 }
 
-STDMETHODIMP CInFileStream::Read(void *data, UInt32 size, UInt32 *processedSize)
+Z7_COM7F_IMF(CInFileStream::Read(void *data, UInt32 size, UInt32 *processedSize))
 {
-  #ifdef USE_WIN_FILE
+  // printf("\nCInFileStream::Read size=%d, VirtPos=%8d\n", (unsigned)size, (int)VirtPos);
+
+  #ifdef Z7_FILE_STREAMS_USE_WIN_FILE
   
-  #ifdef SUPPORT_DEVICE_FILE
+  #ifdef Z7_DEVICE_FILE
   if (processedSize)
     *processedSize = 0;
   if (size == 0)
@@ -96,7 +112,7 @@ STDMETHODIMP CInFileStream::Read(void *data, UInt32 size, UInt32 *processedSize)
     {
       if (VirtPos >= File.Size)
         return VirtPos == File.Size ? S_OK : E_FAIL;
-      UInt64 rem = File.Size - VirtPos;
+      const UInt64 rem = File.Size - VirtPos;
       if (size > rem)
         size = (UInt32)rem;
     }
@@ -104,13 +120,13 @@ STDMETHODIMP CInFileStream::Read(void *data, UInt32 size, UInt32 *processedSize)
     {
       const UInt32 mask = kClusterSize - 1;
       const UInt64 mask2 = ~(UInt64)mask;
-      UInt64 alignedPos = VirtPos & mask2;
+      const UInt64 alignedPos = VirtPos & mask2;
       if (BufSize > 0 && BufStartPos == alignedPos)
       {
-        UInt32 pos = (UInt32)VirtPos & mask;
+        const UInt32 pos = (UInt32)VirtPos & mask;
         if (pos >= BufSize)
           return S_OK;
-        UInt32 rem = MyMin(BufSize - pos, size);
+        const UInt32 rem = MyMin(BufSize - pos, size);
         memcpy(data, Buf + pos, rem);
         VirtPos += rem;
         if (processedSize)
@@ -119,7 +135,7 @@ STDMETHODIMP CInFileStream::Read(void *data, UInt32 size, UInt32 *processedSize)
       }
       
       bool useBuf = false;
-      if ((VirtPos & mask) != 0 || ((ptrdiff_t)data & mask) != 0 )
+      if ((VirtPos & mask) != 0 || ((size_t)(ptrdiff_t)data & mask) != 0 )
         useBuf = true;
       else
       {
@@ -138,7 +154,7 @@ STDMETHODIMP CInFileStream::Read(void *data, UInt32 size, UInt32 *processedSize)
       if (alignedPos != PhyPos)
       {
         UInt64 realNewPosition;
-        bool result = File.Seek((Int64)alignedPos, FILE_BEGIN, realNewPosition);
+        const bool result = File.Seek((Int64)alignedPos, FILE_BEGIN, realNewPosition);
         if (!result)
           return ConvertBoolToHRESULT(result);
         PhyPos = realNewPosition;
@@ -155,7 +171,7 @@ STDMETHODIMP CInFileStream::Read(void *data, UInt32 size, UInt32 *processedSize)
         if (!Buf)
           return E_OUTOFMEMORY;
       }
-      bool result = File.Read1(Buf, readSize, BufSize);
+      const bool result = File.Read1(Buf, readSize, BufSize);
       if (!result)
         return ConvertBoolToHRESULT(result);
 
@@ -180,7 +196,7 @@ STDMETHODIMP CInFileStream::Read(void *data, UInt32 size, UInt32 *processedSize)
   if (processedSize)
     *processedSize = realProcessedSize;
 
-  #ifdef SUPPORT_DEVICE_FILE
+  #ifdef Z7_DEVICE_FILE
   VirtPos += realProcessedSize;
   PhyPos += realProcessedSize;
   #endif
@@ -188,7 +204,7 @@ STDMETHODIMP CInFileStream::Read(void *data, UInt32 size, UInt32 *processedSize)
   if (result)
     return S_OK;
 
-  #else // USE_WIN_FILE
+  #else // Z7_FILE_STREAMS_USE_WIN_FILE
   
   if (processedSize)
     *processedSize = 0;
@@ -199,10 +215,14 @@ STDMETHODIMP CInFileStream::Read(void *data, UInt32 size, UInt32 *processedSize)
       *processedSize = (UInt32)res;
     return S_OK;
   }
-  #endif // USE_WIN_FILE
+  #endif // Z7_FILE_STREAMS_USE_WIN_FILE
 
   {
     const DWORD error = ::GetLastError();
+#if 0
+    if (File.IsStdStream && error == ERROR_BROKEN_PIPE)
+      return S_OK; // end of stream
+#endif
     if (Callback)
       return Callback->InFileStream_On_Error(CallbackRef, error);
     if (error == 0)
@@ -212,7 +232,7 @@ STDMETHODIMP CInFileStream::Read(void *data, UInt32 size, UInt32 *processedSize)
 }
 
 #ifdef UNDER_CE
-STDMETHODIMP CStdInFileStream::Read(void *data, UInt32 size, UInt32 *processedSize)
+Z7_COM7F_IMF(CStdInFileStream::Read(void *data, UInt32 size, UInt32 *processedSize))
 {
   size_t s2 = fread(data, 1, size, stdin);
   int error = ferror(stdin);
@@ -223,15 +243,37 @@ STDMETHODIMP CStdInFileStream::Read(void *data, UInt32 size, UInt32 *processedSi
   return E_FAIL;
 }
 #else
-STDMETHODIMP CStdInFileStream::Read(void *data, UInt32 size, UInt32 *processedSize)
+Z7_COM7F_IMF(CStdInFileStream::Read(void *data, UInt32 size, UInt32 *processedSize))
 {
+  // printf("\nCStdInFileStream::Read size = %d\n", (unsigned)size);
   #ifdef _WIN32
   
   DWORD realProcessedSize;
   UInt32 sizeTemp = (1 << 20);
   if (sizeTemp > size)
     sizeTemp = size;
+  /* in GUI mode : GetStdHandle(STD_INPUT_HANDLE) returns NULL,
+     and it doesn't set LastError.  */
+  /*
+  SetLastError(0);
+  const HANDLE h = GetStdHandle(STD_INPUT_HANDLE);
+  if (!h || h == INVALID_HANDLE_VALUE)
+  {
+    if (processedSize)
+      *processedSize = 0;
+    if (GetLastError() == 0)
+      SetLastError(ERROR_INVALID_HANDLE);
+    return GetLastError_noZero_HRESULT();
+  }
+  */
   BOOL res = ::ReadFile(GetStdHandle(STD_INPUT_HANDLE), data, sizeTemp, &realProcessedSize, NULL);
+
+  /*
+  printf("\nCInFileStream::Read: size=%d, processed=%8d res=%d 4rror=%3d\n",
+    (unsigned)size, (int)realProcessedSize,
+    (int)res, GetLastError());
+  */
+
   if (processedSize)
     *processedSize = realProcessedSize;
   if (res == FALSE && GetLastError() == ERROR_BROKEN_PIPE)
@@ -259,14 +301,68 @@ STDMETHODIMP CStdInFileStream::Read(void *data, UInt32 size, UInt32 *processedSi
   
 #endif
 
-STDMETHODIMP CInFileStream::Seek(Int64 offset, UInt32 seekOrigin, UInt64 *newPosition)
+
+/*
+bool CreateStdInStream(CMyComPtr<ISequentialInStream> &str)
 {
+#if 0
+  CInFileStream *inStreamSpec = new CInFileStream;
+  CMyComPtr<ISequentialInStream> inStreamLoc(inStreamSpec);;
+  if (!inStreamSpec->OpenStdIn())
+    return false;
+  if (!inStreamSpec->File.IsStdPipeStream)
+    str = inStreamLoc.Detach();
+  else
+#endif
+  str = new CStdInFileStream;
+  return true;
+}
+*/
+
+#if 0
+bool CInFileStream::OpenStdIn()
+{
+  _info_WasLoaded = false;
+  // Sleep(100);
+  bool res = File.AttachStdIn();
+  if (!res)
+    return false;
+#if 1
+  CStreamFileProps props;
+  if (GetProps2(&props) != S_OK)
+  {
+    // we can ignore that error
+    return false;
+  }
+  // we can't use Size, because Size can be set for pipe streams for some value.
+  // Seek() sees only current chunk in pipe buffer.
+  // So Seek() can move across only current unread chunk.
+  // But after reading that chunk. it can't move position back.
+  // We need safe check that shows that we can use seek (non-pipe mode)
+  // Is it safe check that shows that pipe mode was used?
+  File.IsStdPipeStream = (props.VolID == 0);
+    // && FILETIME_IsZero(props.CTime)
+    // && FILETIME_IsZero(props.ATime)
+    // && FILETIME_IsZero(props.MTime);
+#endif
+  // printf("\n######## pipe=%d", (unsigned)File.IsStdPipeStream);
+  return true;
+}
+#endif
+
+
+Z7_COM7F_IMF(CInFileStream::Seek(Int64 offset, UInt32 seekOrigin, UInt64 *newPosition))
+{
+  /*
+  printf("\nCInFileStream::Seek seekOrigin=%d, offset=%8d, VirtPos=%8d\n",
+      (unsigned)seekOrigin, (int)offset, (int)VirtPos);
+  */
   if (seekOrigin >= 3)
     return STG_E_INVALIDFUNCTION;
 
-  #ifdef USE_WIN_FILE
+  #ifdef Z7_FILE_STREAMS_USE_WIN_FILE
 
-  #ifdef SUPPORT_DEVICE_FILE
+  #ifdef Z7_DEVICE_FILE
   if (File.IsDeviceFile && (File.SizeDefined || seekOrigin != STREAM_SEEK_END))
   {
     switch (seekOrigin)
@@ -293,7 +389,7 @@ STDMETHODIMP CInFileStream::Seek(Int64 offset, UInt32 seekOrigin, UInt64 *newPos
      in case of error. So we don't need additional code below */
   // if (!result) { realNewPosition = 0; File.GetPosition(realNewPosition); }
   
-  #ifdef SUPPORT_DEVICE_FILE
+  #ifdef Z7_DEVICE_FILE
   PhyPos = VirtPos = realNewPosition;
   #endif
 
@@ -319,17 +415,19 @@ STDMETHODIMP CInFileStream::Seek(Int64 offset, UInt32 seekOrigin, UInt64 *newPos
   #endif
 }
 
-STDMETHODIMP CInFileStream::GetSize(UInt64 *size)
+Z7_COM7F_IMF(CInFileStream::GetSize(UInt64 *size))
 {
   return ConvertBoolToHRESULT(File.GetLength(*size));
 }
 
-#ifdef USE_WIN_FILE
+#ifdef Z7_FILE_STREAMS_USE_WIN_FILE
 
-STDMETHODIMP CInFileStream::GetProps(UInt64 *size, FILETIME *cTime, FILETIME *aTime, FILETIME *mTime, UInt32 *attrib)
+Z7_COM7F_IMF(CInFileStream::GetProps(UInt64 *size, FILETIME *cTime, FILETIME *aTime, FILETIME *mTime, UInt32 *attrib))
 {
   if (!_info_WasLoaded)
-    RINOK(ReloadProps());
+  {
+    RINOK(ReloadProps())
+  }
   const BY_HANDLE_FILE_INFORMATION &info = _info;
   /*
   BY_HANDLE_FILE_INFORMATION info;
@@ -346,10 +444,12 @@ STDMETHODIMP CInFileStream::GetProps(UInt64 *size, FILETIME *cTime, FILETIME *aT
   }
 }
 
-STDMETHODIMP CInFileStream::GetProps2(CStreamFileProps *props)
+Z7_COM7F_IMF(CInFileStream::GetProps2(CStreamFileProps *props))
 {
   if (!_info_WasLoaded)
-    RINOK(ReloadProps());
+  {
+    RINOK(ReloadProps())
+  }
   const BY_HANDLE_FILE_INFORMATION &info = _info;
   /*
   BY_HANDLE_FILE_INFORMATION info;
@@ -370,17 +470,19 @@ STDMETHODIMP CInFileStream::GetProps2(CStreamFileProps *props)
   }
 }
 
-STDMETHODIMP CInFileStream::GetProperty(PROPID propID, PROPVARIANT *value)
+Z7_COM7F_IMF(CInFileStream::GetProperty(PROPID propID, PROPVARIANT *value))
 {
   if (!_info_WasLoaded)
-    RINOK(ReloadProps());
+  {
+    RINOK(ReloadProps())
+  }
 
   if (!_info_WasLoaded)
     return S_OK;
 
   NWindows::NCOM::CPropVariant prop;
 
- #ifdef SUPPORT_DEVICE_FILE
+ #ifdef Z7_DEVICE_FILE
   if (File.IsDeviceFile)
   {
     switch (propID)
@@ -436,9 +538,9 @@ STDMETHODIMP CInFileStream::GetProperty(PROPID propID, PROPVARIANT *value)
 }
 
 
-STDMETHODIMP CInFileStream::ReloadProps()
+Z7_COM7F_IMF(CInFileStream::ReloadProps())
 {
- #ifdef SUPPORT_DEVICE_FILE
+ #ifdef Z7_DEVICE_FILE
   if (File.IsDeviceFile)
   {
     memset(&_info, 0, sizeof(_info));
@@ -455,16 +557,44 @@ STDMETHODIMP CInFileStream::ReloadProps()
   _info_WasLoaded = File.GetFileInformation(&_info);
   if (!_info_WasLoaded)
     return GetLastError_HRESULT();
+#ifdef _WIN32
+#if 0
+  printf(
+    "\ndwFileAttributes = %8x"
+    "\nftCreationTime   = %8x"
+    "\nftLastAccessTime = %8x"
+    "\nftLastWriteTime  = %8x"
+    "\ndwVolumeSerialNumber  = %8x"
+    "\nnFileSizeHigh  = %8x"
+    "\nnFileSizeLow   = %8x"
+    "\nnNumberOfLinks  = %8x"
+    "\nnFileIndexHigh  = %8x"
+    "\nnFileIndexLow   = %8x \n",
+      (unsigned)_info.dwFileAttributes,
+      (unsigned)_info.ftCreationTime.dwHighDateTime,
+      (unsigned)_info.ftLastAccessTime.dwHighDateTime,
+      (unsigned)_info.ftLastWriteTime.dwHighDateTime,
+      (unsigned)_info.dwVolumeSerialNumber,
+      (unsigned)_info.nFileSizeHigh,
+      (unsigned)_info.nFileSizeLow,
+      (unsigned)_info.nNumberOfLinks,
+      (unsigned)_info.nFileIndexHigh,
+      (unsigned)_info.nFileIndexLow);
+#endif
+#endif
   return S_OK;
 }
 
 
 #elif !defined(_WIN32)
 
-STDMETHODIMP CInFileStream::GetProps(UInt64 *size, FILETIME *cTime, FILETIME *aTime, FILETIME *mTime, UInt32 *attrib)
+Z7_COM7F_IMF(CInFileStream::GetProps(UInt64 *size, FILETIME *cTime, FILETIME *aTime, FILETIME *mTime, UInt32 *attrib))
 {
+  // printf("\nCInFileStream::GetProps VirtPos = %8d\n", (int)VirtPos);
   if (!_info_WasLoaded)
-    RINOK(ReloadProps());
+  {
+    RINOK(ReloadProps())
+  }
   const struct stat &st = _info;
   /*
   struct stat st;
@@ -483,10 +613,13 @@ STDMETHODIMP CInFileStream::GetProps(UInt64 *size, FILETIME *cTime, FILETIME *aT
 
 // #include <stdio.h>
 
-STDMETHODIMP CInFileStream::GetProps2(CStreamFileProps *props)
+Z7_COM7F_IMF(CInFileStream::GetProps2(CStreamFileProps *props))
 {
+  // printf("\nCInFileStream::GetProps2 VirtPos = %8d\n", (int)VirtPos);
   if (!_info_WasLoaded)
-    RINOK(ReloadProps());
+  {
+    RINOK(ReloadProps())
+  }
   const struct stat &st = _info;
   /*
   struct stat st;
@@ -521,10 +654,13 @@ STDMETHODIMP CInFileStream::GetProps2(CStreamFileProps *props)
   return S_OK;
 }
 
-STDMETHODIMP CInFileStream::GetProperty(PROPID propID, PROPVARIANT *value)
+Z7_COM7F_IMF(CInFileStream::GetProperty(PROPID propID, PROPVARIANT *value))
 {
+  // printf("\nCInFileStream::GetProperty VirtPos = %8d propID = %3d\n", (int)VirtPos, propID);
   if (!_info_WasLoaded)
-    RINOK(ReloadProps());
+  {
+    RINOK(ReloadProps())
+  }
 
   if (!_info_WasLoaded)
     return S_OK;
@@ -544,6 +680,11 @@ STDMETHODIMP CInFileStream::GetProperty(PROPID propID, PROPVARIANT *value)
       case kpidMTime:  PropVariant_SetFrom_FiTime(prop, ST_MTIME(st)); break;
       case kpidPosixAttrib: prop = (UInt32)st.st_mode; break;
 
+        #if defined(__APPLE__)
+        #pragma GCC diagnostic push
+        #pragma GCC diagnostic ignored "-Wsign-conversion"
+        #endif
+
       case kpidDeviceMajor:
       {
         // printf("\nst.st_rdev = %d\n", st.st_rdev);
@@ -563,6 +704,10 @@ STDMETHODIMP CInFileStream::GetProperty(PROPID propID, PROPVARIANT *value)
         // prop = (UInt32)123456789; // for debug
         break;
 
+        #if defined(__APPLE__)
+        #pragma GCC diagnostic pop
+        #endif
+
       /*
       case kpidDevice:
         if (S_ISCHR(st.st_mode) ||
@@ -625,6 +770,7 @@ STDMETHODIMP CInFileStream::GetProperty(PROPID propID, PROPVARIANT *value)
         }
         break;
       }
+      default: break;
     }
   }
   prop.Detach(value);
@@ -632,7 +778,7 @@ STDMETHODIMP CInFileStream::GetProperty(PROPID propID, PROPVARIANT *value)
 }
 
 
-STDMETHODIMP CInFileStream::ReloadProps()
+Z7_COM7F_IMF(CInFileStream::ReloadProps())
 {
   _info_WasLoaded = (File.my_fstat(&_info) == 0);
   if (!_info_WasLoaded)
@@ -653,9 +799,9 @@ HRESULT COutFileStream::Close()
   return ConvertBoolToHRESULT(File.Close());
 }
 
-STDMETHODIMP COutFileStream::Write(const void *data, UInt32 size, UInt32 *processedSize)
+Z7_COM7F_IMF(COutFileStream::Write(const void *data, UInt32 size, UInt32 *processedSize))
 {
-  #ifdef USE_WIN_FILE
+  #ifdef Z7_FILE_STREAMS_USE_WIN_FILE
 
   UInt32 realProcessedSize;
   const bool result = File.Write(data, size, realProcessedSize);
@@ -680,12 +826,12 @@ STDMETHODIMP COutFileStream::Write(const void *data, UInt32 size, UInt32 *proces
   #endif
 }
   
-STDMETHODIMP COutFileStream::Seek(Int64 offset, UInt32 seekOrigin, UInt64 *newPosition)
+Z7_COM7F_IMF(COutFileStream::Seek(Int64 offset, UInt32 seekOrigin, UInt64 *newPosition))
 {
   if (seekOrigin >= 3)
     return STG_E_INVALIDFUNCTION;
   
-  #ifdef USE_WIN_FILE
+  #ifdef Z7_FILE_STREAMS_USE_WIN_FILE
 
   UInt64 realNewPosition = 0;
   const bool result = File.Seek(offset, seekOrigin, realNewPosition);
@@ -705,7 +851,7 @@ STDMETHODIMP COutFileStream::Seek(Int64 offset, UInt32 seekOrigin, UInt64 *newPo
   #endif
 }
 
-STDMETHODIMP COutFileStream::SetSize(UInt64 newSize)
+Z7_COM7F_IMF(COutFileStream::SetSize(UInt64 newSize))
 {
   return ConvertBoolToHRESULT(File.SetLength_KeepPosition(newSize));
 }
@@ -717,7 +863,7 @@ HRESULT COutFileStream::GetSize(UInt64 *size)
 
 #ifdef UNDER_CE
 
-STDMETHODIMP CStdOutFileStream::Write(const void *data, UInt32 size, UInt32 *processedSize)
+Z7_COM7F_IMF(CStdOutFileStream::Write(const void *data, UInt32 size, UInt32 *processedSize))
 {
   size_t s2 = fwrite(data, 1, size, stdout);
   if (processedSize)
@@ -727,7 +873,7 @@ STDMETHODIMP CStdOutFileStream::Write(const void *data, UInt32 size, UInt32 *pro
 
 #else
 
-STDMETHODIMP CStdOutFileStream::Write(const void *data, UInt32 size, UInt32 *processedSize)
+Z7_COM7F_IMF(CStdOutFileStream::Write(const void *data, UInt32 size, UInt32 *processedSize))
 {
   if (processedSize)
     *processedSize = 0;
