@@ -1,7 +1,7 @@
 // UpdateCallbackConsole.h
 
-#ifndef __UPDATE_CALLBACK_CONSOLE_H
-#define __UPDATE_CALLBACK_CONSOLE_H
+#ifndef ZIP7_INC_UPDATE_CALLBACK_CONSOLE_H
+#define ZIP7_INC_UPDATE_CALLBACK_CONSOLE_H
 
 #include "../../../Common/StdOutStream.h"
 
@@ -26,32 +26,34 @@ struct CErrorPathCodes
   }
 };
 
+
 class CCallbackConsoleBase
 {
-protected:
-  CPercentPrinter _percent;
+  void CommonError(const FString &path, DWORD systemError, bool isWarning);
 
+protected:
   CStdOutStream *_so;
   CStdOutStream *_se;
 
-  void CommonError(const FString &path, DWORD systemError, bool isWarning);
-  // void CommonError(const char *message);
-
   HRESULT ScanError_Base(const FString &path, DWORD systemError);
   HRESULT OpenFileError_Base(const FString &name, DWORD systemError);
   HRESULT ReadingFileError_Base(const FString &name, DWORD systemError);
 
 public:
-  bool NeedPercents() const { return _percent._so != NULL; };
-
   bool StdOutMode;
-
   bool NeedFlush;
   unsigned PercentsNameLevel;
   unsigned LogLevel;
 
+protected:
   AString _tempA;
   UString _tempU;
+  CPercentPrinter _percent;
+
+public:
+  CErrorPathCodes FailedFiles;
+  CErrorPathCodes ScanErrors;
+  UInt64 NumNonOpenFiles;
 
   CCallbackConsoleBase():
       StdOutMode(false),
@@ -61,15 +63,21 @@ public:
       NumNonOpenFiles(0)
       {}
   
+  bool NeedPercents() const { return _percent._so != NULL; }
   void SetWindowWidth(unsigned width) { _percent.MaxLen = width - 1; }
 
-  void Init(CStdOutStream *outStream, CStdOutStream *errorStream, CStdOutStream *percentStream)
+  void Init(
+      CStdOutStream *outStream,
+      CStdOutStream *errorStream,
+      CStdOutStream *percentStream,
+      bool disablePercents)
   {
     FailedFiles.Clear();
 
     _so = outStream;
     _se = errorStream;
     _percent._so = percentStream;
+    _percent.DisablePrint = disablePercents;
   }
 
   void ClosePercents2()
@@ -84,39 +92,50 @@ public:
       _percent.ClosePrint(false);
   }
 
-  CErrorPathCodes FailedFiles;
-  CErrorPathCodes ScanErrors;
-  UInt64 NumNonOpenFiles;
-
   HRESULT PrintProgress(const wchar_t *name, bool isDir, const char *command, bool showInLog);
 
   // void PrintInfoLine(const UString &s);
   // void PrintPropInfo(UString &s, PROPID propID, const PROPVARIANT *value);
 };
 
-class CUpdateCallbackConsole: public IUpdateCallbackUI2, public CCallbackConsoleBase
+
+class CUpdateCallbackConsole Z7_final:
+  public IUpdateCallbackUI2,
+  public CCallbackConsoleBase
 {
   // void PrintPropPair(const char *name, const wchar_t *val);
+  Z7_IFACE_IMP(IUpdateCallbackUI)
+  Z7_IFACE_IMP(IDirItemsCallback)
+  Z7_IFACE_IMP(IUpdateCallbackUI2)
+
+  HRESULT MoveArc_UpdateStatus();
+
+  UInt64 _arcMoving_total;
+  UInt64 _arcMoving_current;
+  UInt64 _arcMoving_percents;
+  Int32  _arcMoving_updateMode;
 
 public:
   bool DeleteMessageWasShown;
 
-  #ifndef _NO_CRYPTO
+  #ifndef Z7_NO_CRYPTO
   bool PasswordIsDefined;
-  UString Password;
   bool AskPassword;
+  UString Password;
   #endif
 
   CUpdateCallbackConsole():
-      DeleteMessageWasShown(false)
-      #ifndef _NO_CRYPTO
+        _arcMoving_total(0)
+      , _arcMoving_current(0)
+      , _arcMoving_percents(0)
+      , _arcMoving_updateMode(0)
+      , DeleteMessageWasShown(false)
+      #ifndef Z7_NO_CRYPTO
       , PasswordIsDefined(false)
       , AskPassword(false)
       #endif
       {}
 
-  virtual ~CUpdateCallbackConsole() {}
-  
   /*
   void Init(CStdOutStream *outStream)
   {
@@ -124,7 +143,6 @@ public:
   }
   */
   // ~CUpdateCallbackConsole() { if (NeedPercents()) _percent.ClosePrint(); }
-  INTERFACE_IUpdateCallbackUI2(;)
 };
 
 #endif
