@@ -1,5 +1,5 @@
 /* HuffEnc.c -- functions for Huffman encoding
-2023-03-04 : Igor Pavlov : Public domain */
+2023-09-07 : Igor Pavlov : Public domain */
 
 #include "Precomp.h"
 
@@ -8,7 +8,7 @@
 
 #define kMaxLen 16
 #define NUM_BITS 10
-#define MASK (((unsigned)1 << NUM_BITS) - 1)
+#define MASK ((1u << NUM_BITS) - 1)
 
 #define NUM_COUNTERS 64
 
