@@ -1,23 +1,23 @@
 // Windows/FileIO.h
 
-#ifndef __WINDOWS_FILE_IO_H
-#define __WINDOWS_FILE_IO_H
+#ifndef ZIP7_INC_WINDOWS_FILE_IO_H
+#define ZIP7_INC_WINDOWS_FILE_IO_H
 
 #include "../Common/MyWindows.h"
 
-#define _my_IO_REPARSE_TAG_MOUNT_POINT  (0xA0000003L)
-#define _my_IO_REPARSE_TAG_SYMLINK      (0xA000000CL)
-#define _my_IO_REPARSE_TAG_LX_SYMLINK   (0xA000001DL)
+#define Z7_WIN_IO_REPARSE_TAG_MOUNT_POINT  (0xA0000003L)
+#define Z7_WIN_IO_REPARSE_TAG_SYMLINK      (0xA000000CL)
+#define Z7_WIN_IO_REPARSE_TAG_LX_SYMLINK   (0xA000001DL)
 
-#define _my_SYMLINK_FLAG_RELATIVE 1
+#define Z7_WIN_SYMLINK_FLAG_RELATIVE 1
 
 // what the meaning of that FLAG or field (2)?
-#define _my_LX_SYMLINK_FLAG 2
+#define Z7_WIN_LX_SYMLINK_FLAG 2
 
 #ifdef _WIN32
 
 #if defined(_WIN32) && !defined(UNDER_CE)
-#include <WinIoCtl.h>
+#include <winioctl.h>
 #endif
 
 #else
@@ -76,11 +76,11 @@ struct CReparseAttr
   // returns (false) and (ErrorCode = ERROR_REPARSE_TAG_INVALID), if unknown tag
   bool Parse(const Byte *p, size_t size);
 
-  bool IsMountPoint()  const { return Tag == _my_IO_REPARSE_TAG_MOUNT_POINT; } // it's Junction
-  bool IsSymLink_Win() const { return Tag == _my_IO_REPARSE_TAG_SYMLINK; }
-  bool IsSymLink_WSL() const { return Tag == _my_IO_REPARSE_TAG_LX_SYMLINK; }
+  bool IsMountPoint()  const { return Tag == Z7_WIN_IO_REPARSE_TAG_MOUNT_POINT; } // it's Junction
+  bool IsSymLink_Win() const { return Tag == Z7_WIN_IO_REPARSE_TAG_SYMLINK; }
+  bool IsSymLink_WSL() const { return Tag == Z7_WIN_IO_REPARSE_TAG_LX_SYMLINK; }
 
-  bool IsRelative_Win() const { return Flags == _my_SYMLINK_FLAG_RELATIVE; }
+  bool IsRelative_Win() const { return Flags == Z7_WIN_SYMLINK_FLAG_RELATIVE; }
 
   bool IsRelative_WSL() const
   {
@@ -141,17 +141,30 @@ public:
 
 public:
   bool PreserveATime;
-  #ifdef SUPPORT_DEVICE_FILE
+#if 0
+  bool IsStdStream;
+  bool IsStdPipeStream;
+#endif
+#ifdef Z7_DEVICE_FILE
   bool IsDeviceFile;
   bool SizeDefined;
   UInt64 Size; // it can be larger than real available size
-  #endif
+#endif
 
-  CFileBase(): _handle(INVALID_HANDLE_VALUE), PreserveATime(false) {};
+  CFileBase():
+    _handle(INVALID_HANDLE_VALUE),
+    PreserveATime(false)
+#if 0
+    , IsStdStream(false),
+    , IsStdPipeStream(false)
+#endif
+    {}
   ~CFileBase() { Close(); }
 
   HANDLE GetHandle() const { return _handle; }
 
+  // void Detach() { _handle = INVALID_HANDLE_VALUE; }
+
   bool Close() throw();
 
   bool GetPosition(UInt64 &position) const throw();
@@ -193,7 +206,7 @@ struct my_DISK_GEOMETRY_EX
 
 class CInFile: public CFileBase
 {
-  #ifdef SUPPORT_DEVICE_FILE
+  #ifdef Z7_DEVICE_FILE
 
   #ifndef UNDER_CE
   
@@ -221,6 +234,20 @@ public:
   bool OpenShared(CFSTR fileName, bool shareForWrite);
   bool Open(CFSTR fileName);
 
+#if 0
+  bool AttachStdIn()
+  {
+    IsDeviceFile = false;
+    const HANDLE h = GetStdHandle(STD_INPUT_HANDLE);
+    if (h == INVALID_HANDLE_VALUE || !h)
+      return false;
+    IsStdStream = true;
+    IsStdPipeStream = true;
+    _handle = h;
+    return true;
+  }
+#endif
+
   #ifndef UNDER_CE
 
   bool Open_for_ReadAttributes(CFSTR fileName)
@@ -232,6 +259,14 @@ public:
     // we must use (FILE_FLAG_BACKUP_SEMANTICS) to open handle of directory.
   }
 
+  bool Open_for_FileRenameInformation(CFSTR fileName)
+  {
+    return Create(fileName, DELETE | SYNCHRONIZE | GENERIC_READ,
+        FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
+        OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL);
+    // we must use (FILE_FLAG_BACKUP_SEMANTICS) to open handle of directory.
+  }
+
   bool OpenReparse(CFSTR fileName)
   {
     // 17.02 fix: to support Windows XP compatibility junctions:
@@ -253,11 +288,21 @@ public:
 
 class COutFile: public CFileBase
 {
+  bool Open_Disposition(CFSTR fileName, DWORD creationDisposition);
 public:
   bool Open(CFSTR fileName, DWORD shareMode, DWORD creationDisposition, DWORD flagsAndAttributes);
-  bool Open(CFSTR fileName, DWORD creationDisposition);
-  bool Create(CFSTR fileName, bool createAlways);
-  bool CreateAlways(CFSTR fileName, DWORD flagsAndAttributes);
+  bool Open_EXISTING(CFSTR fileName)
+    { return Open_Disposition(fileName, OPEN_EXISTING); }
+  bool Create_ALWAYS_or_Open_ALWAYS(CFSTR fileName, bool createAlways)
+    { return Open_Disposition(fileName, createAlways ? CREATE_ALWAYS : OPEN_ALWAYS); }
+  bool Create_ALWAYS_or_NEW(CFSTR fileName, bool createAlways)
+    { return Open_Disposition(fileName, createAlways ? CREATE_ALWAYS : CREATE_NEW); }
+  bool Create_ALWAYS(CFSTR fileName)
+    { return Open_Disposition(fileName, CREATE_ALWAYS); }
+  bool Create_NEW(CFSTR fileName)
+    { return Open_Disposition(fileName, CREATE_NEW); }
+  
+  bool Create_ALWAYS_with_Attribs(CFSTR fileName, DWORD flagsAndAttributes);
 
   bool SetTime(const CFiTime *cTime, const CFiTime *aTime, const CFiTime *mTime) throw();
   bool SetMTime(const CFiTime *mTime) throw();
@@ -295,12 +340,20 @@ protected:
   UInt64 Size; // it can be larger than real available size
   */
 
-  bool OpenBinary(const char *name, int flags);
+  bool OpenBinary(const char *name, int flags, mode_t mode = 0666);
 public:
   bool PreserveATime;
+#if 0
+  bool IsStdStream;
+#endif
 
-  CFileBase(): _handle(-1), PreserveATime(false) {};
+  CFileBase(): _handle(-1), PreserveATime(false)
+#if 0
+    , IsStdStream(false)
+#endif
+  {}
   ~CFileBase() { Close(); }
+  // void Detach() { _handle = -1; }
   bool Close();
   bool GetLength(UInt64 &length) const;
   off_t seek(off_t distanceToMove, int moveMethod) const;
@@ -320,6 +373,15 @@ class CInFile: public CFileBase
 public:
   bool Open(const char *name);
   bool OpenShared(const char *name, bool shareForWrite);
+#if 0
+  bool AttachStdIn()
+  {
+    _handle = GetStdHandle(STD_INPUT_HANDLE);
+    if (_handle == INVALID_HANDLE_VALUE || !_handle)
+      return false;
+    IsStdStream = true;
+  }
+#endif
   ssize_t read_part(void *data, size_t size) throw();
   // ssize_t read_full(void *data, size_t size, size_t &processed);
   bool ReadFull(void *data, size_t size, size_t &processedSize) throw();
@@ -330,23 +392,32 @@ class COutFile: public CFileBase
   bool CTime_defined;
   bool ATime_defined;
   bool MTime_defined;
-
   CFiTime CTime;
   CFiTime ATime;
   CFiTime MTime;
 
   AString Path;
   ssize_t write_part(const void *data, size_t size) throw();
+  bool OpenBinary_forWrite_oflag(const char *name, int oflag);
 public:
+  mode_t mode_for_Create;
+
   COutFile():
       CTime_defined(false),
       ATime_defined(false),
-      MTime_defined(false)
+      MTime_defined(false),
+      mode_for_Create(0666)
       {}
 
   bool Close();
-  bool Create(const char *name, bool createAlways);
-  bool Open(const char *name, DWORD creationDisposition);
+
+  bool Open_EXISTING(CFSTR fileName);
+  bool Create_ALWAYS_or_Open_ALWAYS(CFSTR fileName, bool createAlways);
+  bool Create_ALWAYS(CFSTR fileName);
+  bool Create_NEW(CFSTR fileName);
+  // bool Create_ALWAYS_or_NEW(CFSTR fileName, bool createAlways);
+  // bool Open_Disposition(const char *name, DWORD creationDisposition);
+
   ssize_t write_full(const void *data, size_t size, size_t &processed) throw();
 
   bool WriteFull(const void *data, size_t size) throw()
